package com.speedingplanet.banking;

import com.fasterxml.jackson.databind.SerializationFeature;
import com.speedingplanet.banking.api.Category;
import com.speedingplanet.banking.api.Payee;
import com.speedingplanet.banking.api.Transaction;
import com.speedingplanet.banking.db.CategoryDAO;
import com.speedingplanet.banking.db.PayeeDAO;
import com.speedingplanet.banking.db.TransactionDAO;
import com.speedingplanet.banking.resources.*;
import io.dropwizard.Application;
import io.dropwizard.assets.AssetsBundle;
import io.dropwizard.db.DataSourceFactory;
import io.dropwizard.hibernate.HibernateBundle;
import io.dropwizard.setup.Bootstrap;
import io.dropwizard.setup.Environment;
import org.eclipse.jetty.servlets.CrossOriginFilter;
import org.glassfish.jersey.filter.LoggingFilter;

import javax.servlet.DispatcherType;
import javax.servlet.FilterRegistration;
import java.util.EnumSet;
import java.util.logging.Logger;

public class BankingApplication extends Application<BankingConfiguration> {

  public static void main(final String[] args) throws Exception {
    new BankingApplication().run(args);
  }

  @Override
  public String getName() {
    return "Banking";
  }

  private final HibernateBundle<BankingConfiguration> hibernateBundle =
      new HibernateBundle<BankingConfiguration>(Transaction.class, Category.class, Payee.class) {
        @Override
        public DataSourceFactory getDataSourceFactory(BankingConfiguration configuration) {
          return configuration.getDataSourceFactory();
        }
      };

  @Override
  public void initialize(final Bootstrap<BankingConfiguration> bootstrap) {
    bootstrap.addBundle(hibernateBundle);
    bootstrap.addBundle(new AssetsBundle("/assets", "/", "index.html"));
  }

  @Override
  public void run(final BankingConfiguration configuration,
                  final Environment environment) {

    configureCors(environment);

    // Pretty-print JSON
    environment.getObjectMapper().enable(SerializationFeature.INDENT_OUTPUT);

    // Logs any incoming request bodies at the INFO level
    environment.jersey().register(new LoggingFilter(Logger.getLogger("InboundRequestResponse"), true));

    final TransactionDAO txDAO = new TransactionDAO(hibernateBundle.getSessionFactory());
    environment.jersey().register(new TransactionResource(txDAO));

    final PayeeDAO payeeDAO = new PayeeDAO(hibernateBundle.getSessionFactory());
    environment.jersey().register(new PayeeResource(payeeDAO));

    final CategoryDAO categoryDAO = new CategoryDAO(hibernateBundle.getSessionFactory());
    environment.jersey().register(new CategoryResource(categoryDAO));

    environment.jersey().register(new HelloWorldResource());
    environment.jersey().register(new GreeterResource("John"));
    environment.jersey().register(new BankingResource());

  }

  private void configureCors(Environment environment) {
    FilterRegistration.Dynamic filter = environment.servlets().addFilter("CORS", new CrossOriginFilter());
    filter.setInitParameter("allowedOrigins", "*");
    filter.setInitParameter("allowedHeaders", "*");
    filter.setInitParameter("allowedHeaders", "*");
    filter.addMappingForUrlPatterns(EnumSet.allOf(DispatcherType.class), true, "/*");
  }

}
