package com.speedingplanet.banking;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.speedingplanet.banking.api.Transaction;
import io.dropwizard.client.JerseyClientBuilder;
import io.dropwizard.client.JerseyClientConfiguration;
import io.dropwizard.db.DataSourceFactory;
import io.dropwizard.jackson.Jackson;
import io.dropwizard.testing.ResourceHelpers;
import io.dropwizard.testing.junit.DropwizardAppRule;
import io.dropwizard.util.Duration;
import org.h2.tools.RunScript;
import org.junit.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.SQLException;

import static io.dropwizard.testing.FixtureHelpers.fixture;
import static org.assertj.core.api.Assertions.assertThat;

public class TransactionInsertUpdateIntegrationTest {
  private static Logger LOGGER = LoggerFactory.getLogger(TransactionInsertUpdateIntegrationTest.class);
  private WebTarget baseTarget;
  private Client client;
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

  // Multiple tests (below) means that this should be a Rule, not a ClassRule
  // per https://github.com/dropwizard/dropwizard/issues/832
  @Rule
  public final DropwizardAppRule<BankingConfiguration> RULE =
      new DropwizardAppRule<>(BankingApplication.class, ResourceHelpers.resourceFilePath("test-config.yml"));

  @Before
  public void setup() throws FileNotFoundException, SQLException {
    JerseyClientConfiguration jcc = new JerseyClientConfiguration();
    jcc.setTimeout(Duration.milliseconds(5000));
    jcc.setConnectionTimeout(Duration.milliseconds(5000));
    client = new JerseyClientBuilder(RULE.getEnvironment())
        .using(jcc)
        .build("test client");

    baseTarget = client.target(String.format("http://localhost:%d/banking", RULE.getLocalPort()));

    DataSourceFactory ds = RULE.getConfiguration().getDataSourceFactory();
    String user = ds.getUser();
    String pass = ds.getPassword();
    String url = ds.getUrl();
    RunScript.execute(url, user, pass, "sql/full-setup.sql", null, false);
  }

  @After
  public void shutdown() {
    client.close();
  }

  @Test
  public void addTx() throws IOException{
    Transaction newTx = MAPPER.readValue(fixture("fixtures/insert-before-tx.json"), Transaction.class);
    Transaction checkTx = insertTx(newTx);

    assertThat(checkTx.getAmount()).isEqualTo(newTx.getAmount());
    assertThat(checkTx.getCategoryId()).isEqualTo(newTx.getCategoryId());
    assertThat(checkTx.getAccountId()).isEqualTo(newTx.getAccountId());
    assertThat(checkTx.getTxDate()).isEqualTo(newTx.getTxDate());
  }

  @Test
  public void updateTx() throws IOException {
    Transaction newTx = MAPPER.readValue(fixture("fixtures/insert-before-tx.json"), Transaction.class);
    Transaction checkTx = insertTx(newTx);

    checkTx.setCategoryId(1);
    checkTx.setAmount(500.0);

    Entity<Transaction> jsonEntity = Entity.entity(checkTx, MediaType.APPLICATION_JSON);
    Response response = baseTarget
        .path("/tx/" + checkTx.getTxId())
        .request(MediaType.APPLICATION_JSON_TYPE)
        .accept(MediaType.APPLICATION_JSON_TYPE)
        .post(jsonEntity);

    Transaction updatedTx = response.readEntity(Transaction.class);
    assertThat(checkTx).isEqualTo(updatedTx);
  }

  @Test
  public void updateThrowsException() throws IOException {
    Transaction checkTx = MAPPER.readValue(fixture("fixtures/transaction.json"), Transaction.class);
    checkTx.setCategoryId(1);
    checkTx.setAmount(500.0);

    Entity<Transaction> jsonEntity = Entity.entity(checkTx, MediaType.APPLICATION_JSON);
    Response response = baseTarget
        .path("/tx/0")
        .request(MediaType.APPLICATION_JSON_TYPE)
        .accept(MediaType.APPLICATION_JSON_TYPE)
        .post(jsonEntity);

    assertThat(response.getStatus()).isEqualTo(Response.Status.BAD_REQUEST.getStatusCode());
  }

  private Transaction insertTx(Transaction inTx) {
    Entity<Transaction> jsonEntity = Entity.entity(inTx, MediaType.APPLICATION_JSON);
    Response response = baseTarget
        .path("/tx")
        .request(MediaType.APPLICATION_JSON_TYPE)
        .accept(MediaType.APPLICATION_JSON_TYPE)
        .post(jsonEntity);
    assertThat(response.getStatusInfo()).isEqualTo(Response.Status.CREATED);
    return response.readEntity(Transaction.class);
  }

}
