package com.speedingplanet.banking.resources;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.CollectionType;
import com.speedingplanet.banking.api.Transaction;
import com.speedingplanet.banking.db.TransactionDAO;
import io.dropwizard.jackson.Jackson;
import io.dropwizard.testing.junit.ResourceTestRule;
import org.joda.time.DateTime;
import org.junit.*;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.runners.MockitoJUnitRunner;

import javax.ws.rs.client.Entity;
import javax.ws.rs.core.*;
import java.io.IOException;
import java.util.List;

import static io.dropwizard.testing.FixtureHelpers.fixture;
import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.*;

@RunWith(MockitoJUnitRunner.class)
public class TransactionResourceTest {
  private static final ObjectMapper MAPPER = Jackson.newObjectMapper();
  private static final TransactionDAO dao = mock(TransactionDAO.class);

  @ClassRule
  public static final ResourceTestRule resources = ResourceTestRule.builder()
      .addResource(new TransactionResource(dao))
      .build();

  @Captor
  ArgumentCaptor<Transaction> txCaptor;

  @Before
  public void setup() {

  }

  @After
  public void tearDown() {
    reset(dao);
  }

  @Test
  public void testGetTransaction() {
    final Transaction tx = new Transaction(1,
        8,
        1,
        "Debit",
        17,
        "Cleared",
        new DateTime("2015-01-04T23:50:19.938-0000"), // Need tz info because otherwise it's EST and fails
        -97.74
    );
    when(dao.findById(eq(1))).thenReturn(tx);
    assertThat(resources.client().target("/tx/1").request().get(Transaction.class))
        .isEqualTo(tx);
    verify(dao).findById(1);
  }

  @Test
  public void testGetTransactions() throws IOException {
    // Mocked data
    CollectionType javaType = MAPPER.getTypeFactory().constructCollectionType(List.class, Transaction.class);
    List<Transaction> txs = MAPPER.readValue(fixture("fixtures/query-tx.json"), javaType);
    MultivaluedMap<String, String> queryParams = new MultivaluedHashMap<String, String>();

    // Mocked response
    queryParams.add("payeeId", "14");
    when(dao.query(queryParams)).thenReturn(txs);

    // Actual test
    // Note use of GenericType<List<Something>>() {} to create the class reference for a List of some type
    assertThat(resources.client().target("/tx/?payeeId=14").request().get(new GenericType<List<Transaction>>() {}))
        .isEqualTo(txs);
    verify(dao).query(queryParams);
  }

  @Test
  public void testInsertTransaction() throws IOException{
    Transaction insertBeforeTx = MAPPER.readValue(fixture("fixtures/insert-before-tx.json"), Transaction.class);

    when(dao.create(any(Transaction.class))).thenReturn(insertBeforeTx);

    Response response = resources
        .client()
        .target("/tx")
        .request(MediaType.APPLICATION_JSON_TYPE)
        .post(Entity.entity(insertBeforeTx, MediaType.APPLICATION_JSON_TYPE));

    assertThat(response.getStatusInfo()).isEqualTo(Response.Status.CREATED);
    verify(dao).create(txCaptor.capture());
    assertThat(txCaptor.getValue()).isEqualTo(insertBeforeTx);
  }

  @Test
  public void testUpdateTransaction() throws IOException{
    Transaction insertAfterTx = MAPPER.readValue(fixture("fixtures/insert-after-tx.json"), Transaction.class);

    when(dao.update(any(Transaction.class))).thenReturn(insertAfterTx);

    Response response = resources
        .client()
        .target("/tx/1000")
        .request(MediaType.APPLICATION_JSON_TYPE)
        .post(Entity.entity(insertAfterTx, MediaType.APPLICATION_JSON_TYPE));

    assertThat(response.getStatusInfo()).isEqualTo(Response.Status.OK);
    verify(dao).update(txCaptor.capture());
    assertThat(txCaptor.getValue()).isEqualTo(insertAfterTx);
  }
}
